//+------------------------------------------------------------------+
//|                                                    GarchTest.mq5 |
//|                                                           denkir |
//|                                                 denkir@gmail.com |
//+------------------------------------------------------------------+
#property copyright "denkir"
#property link      "denkir@gmail.com"
#property version   "1.00"
#property script_show_inputs
// included files
//#include <Complex_class.mqh>    <FFT_class.mqh>
#include <FFT_class.mqh>
#include <Trade\SymbolInfo.mqh>
#include <Google_Charts.mqh> 

input int X_size=700; //    X  px
input int Y_size=250; //    Y  px

                      // definitions
#define ITMAX 100 //maximum allowed number of iterations
#define EPS 3.0e-7 //relative accuracy
#define FPMIN 1.0e-30 //number near the smallest representable double number

                      //functions declarations
void gser(double &gamser,double a,double x,double &gln);
void gcf(double &gammcf,double a,double x,double &gln);
void nrerror(string error_text);
double gammp(double a,double x);
double gammln(double xx);

void autocorr(double &ACF[],double &res[],int nLags);//(1- )
void autocorr(double &ACF[],double &bounds[],int &lags[],double &res[],int nLags);//(2- )
void lbqtest(bool &H[],double &rets[]);
void cumsum(double &sum[],double &x[]);
//void lbqtestInfo(bool &h1[],bool &h2[],double &ACF[],double &bounds[],int &lags[]);
void lbqtestInfo(bool &h1[],bool &h2[],double &rets[],double &retsq[],int &lags[]);
bool any(bool &h[]);

//===
int hRI,//ReturnsIndicator handle
start,  //start time
finish; //finish time
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
   start=GetTickCount(); //  ,   
   ObjectDelete(0,"Chart1");
   if(!ObjectCreate(0,"Chart1",OBJ_BITMAP_LABEL,0,0,0))
     {Alert("  Chart1  "); return;}
   ObjectSetInteger(0,"Chart1",OBJPROP_XDISTANCE,0);
   ObjectSetInteger(0,"Chart1",OBJPROP_YDISTANCE,0);

   ObjectDelete(0,"Chart2");
   if(!ObjectCreate(0,"Chart2",OBJ_BITMAP_LABEL,0,0,0))
     {Alert("  Chart2  "); return;}
   ObjectSetInteger(0,"Chart2",OBJPROP_XDISTANCE,0);
   ObjectSetInteger(0,"Chart2",OBJPROP_YDISTANCE,Y_size+25);

   int N=SeriesInfoInteger(Symbol(),0,SERIES_BARS_COUNT);
   MqlRates rates[]; ArraySetAsSeries(rates,true);
   double nClose1[];
   ArrayResize(nClose1,N);
   int copied=CopyRates(Symbol(),0,0,N,rates);
   if(copied>0)
      Print(" : "+copied);
   else
      Print("      ",Symbol());
   for(int i=0;i<N;i++)
     {
      nClose1[i]=rates[i].close;
     }
   ArraySetAsSeries(nClose1,true);
// ACF
   double rets1[],//rets1 -    N,
   rets[],        //rets - -  N-1
   retsq[];       //rets - -   N-1
   ArrayResize(rets1,N);ArrayResize(rets,N-1);ArrayResize(retsq,N-1);
   hRI=iCustom(NULL,0,"ReturnsIndicator");
   if(CopyBuffer(hRI,0,0,N,rets1)<1)
     {
      Print("    !");
      return;
     }
   if(ArrayCopy(rets,rets1,0,1,N-1)<1)
     {
      Print("    !");
      return;
     }
   bool h1[3],h2[3]; //  
   double m = mean(rets);
   for(int t=0;t<ArraySize(rets);t++)
     {
      rets[t]=rets[t]-m;
      retsq[t]=pow(rets[t],2);
     }
   lbqtest(h1,rets); //Q-   
   lbqtest(h2,retsq);//Q-    
   int lags[];
   lbqtestInfo(h1,h2,rets,retsq,lags); //   
                                       //finish=GetTickCount()-start; Print("Time ",finish," msec"); // 
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void lbqtest(bool &H[],double &rets[])
/* ,  Q- --
   H -    ,   
        ;
   rets -     ;
*/
  {
   double lags[3]={10.0,15.0,20.0};       //  
   int maxLags=20;                        //  
   double ACF[]; ArrayResize(ACF,21);     //  
   double acf[]; ArrayResize(acf,20);     //  
   autocorr(ACF,rets,maxLags);            //  
   for(int i=0;i<20;i++) acf[i]=ACF[i+1]; //   - , 
                                          //-                            
   double alpha[3]={0.05,0.05,0.05};      //   

/*  Q-   
   :            
                  L
                |----|
                 \
       Q = T(T+2) || (rho(k)^2/(T-k)), 
                 /  
                |----| 
                  k=1
   T -  , L -  , rho(k) -    k- .
*/

   double idx[]; ArrayResize(idx,maxLags);     //  
   int len=ArraySize(rets);                    //  
   int arrLags[];ArrayResize(arrLags,maxLags); //  
   double stat[]; ArrayResize(stat,maxLags);   // Q-
   double sum[]; ArrayResize(sum,maxLags);     //  
   double iACF[];ArrayResize(iACF,maxLags);    //  
   for(int i=0;i<maxLags;i++)
     {                                         //:
      arrLags[i]=i+1;                          //       
      idx[i]=len-arrLags[i];                   //        
      iACF[i]=pow(acf[i],2)/idx[i];            //       
     }
   cumsum(sum,iACF);                           //    
                                               //                         
   for(int i=0;i<maxLags;i++)
      stat[i]=sum[i]*len*(len+2);              //  Q-: stat[i]=len*(len+2)*sum[i]; ???
   double stat1[];                             //  Q-
   ArrayResize(stat1,ArraySize(lags));
   for(int i=0;i<ArraySize(lags);i++)
      stat1[i]=stat[lags[i]-1];                // -   

   double pValue[ArraySize(lags)];             // p-
   for(int i=0;i<ArraySize(lags);i++)
     {
      pValue[i]=1-gammp(lags[i]/2,stat1[i]/2); // p-
      H[i]=alpha[i]>=pValue[i];                //  
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void lagmatrix(double &Y[],double &X[],int lag)
//    
  {
   int len=ArraySize(X);
   ArrayResize(Y,len);
   ArrayInitialize(Y,MathArcsin(2.0)); // NaN
   if(lag>=0)
      for(int t=0;t<len-lag;t++)
         Y[t+lag]=X[t];
   else
   for(int t=0;t<len+lag;t++)
             Y[t]=X[t-lag];
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void cumsum(double &sum[],double &x[])
//                 
  {
   sum[0]=x[0];
   for(int i=1;i<ArraySize(x);i++)
      sum[i]=x[i]+sum[i-1];
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void autocorr(double &ACF[],double &res[],int nLags) //1- 
/*     ()      
   ACF -      ;
   res -     ;
   nLags -   ,    .
*/
  {
   Complex Data1[],Data21[], //   
   Data2[],Data22[],         //   
   cData[];                  //              
   double rA[][2];           //    

   int nFFT=pow(2,nextpow2(ArraySize(res))+1);      //  
   ArrayResize(rA,nFFT);ArrayResize(Data1,nFFT);    //  
   ArrayResize(Data2,nFFT);ArrayResize(Data21,nFFT);
   ArrayResize(Data22,nFFT);ArrayResize(cData,nFFT);

   double rets1[];                   //    
   double m=mean(res);               //   res
   ArrayResize(rets1,nFFT);          //  
   for(int t=0;t<ArraySize(res);t++) //    
      //      
      rets1[t]=res[t]-m;
   setComplexArr(Data1,rets1); //       
   CFFT F,F1;                  //   CFFT
   F.setCFFT(Data1,Data2);     // -   F
   F.Forward(nFFT);            //   
   for(int i=0;i<nFFT;i++)
     {
      Data21[i].opEqual(F.Output[i]);//  Data21   F.Output;
      cData[i].conjugate(Data21[i]); //     Data21
      Data21[i].opMultEq(cData[i]);  //      
                                     //  ,     
     }
   F1.setCFFT(Data21,Data22);   // -   F1   
   F1.InverseT(nFFT);           //    
   getComplexArr(rA,F1.Output); //   double  
                                //      
   for(int i=0;i<nLags+1;i++)
     {
      ACF[i]=rA[i][0];          //    ACF  
                                // 
      ACF[i]=ACF[i]/rA[0][0];   //   
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void autocorr(double &ACF[],double &bounds[],int &lags[],double &res[],int nLags) //2- 
/*     ()      
   ACF -      ;
   bounds -      ;
   lags -  ; 
   res -     ;
   nLags -   ,    .
*/
  {
   Complex Data1[],Data21[], //   
   Data2[],Data22[],         //   
   cData[];                  //              
   double rA[][2];           //    

   int nFFT=pow(2,nextpow2(ArraySize(res))+1);      //  
   ArrayResize(rA,nFFT);ArrayResize(Data1,nFFT);    //    nFFT
   ArrayResize(Data2,nFFT);ArrayResize(Data21,nFFT);
   ArrayResize(Data22,nFFT);ArrayResize(cData,nFFT);

   double rets1[];                   //    
   double m=mean(res);               //   res
   ArrayResize(rets1,nFFT);          //  
   for(int t=0;t<ArraySize(res);t++) //    
      //      
      rets1[t]=res[t]-m;
   setComplexArr(Data1,rets1); //       
   CFFT F,F1;                  //   CFFT
   F.setCFFT(Data1,Data2);     // -   F
   F.Forward(nFFT);            //   
   for(int i=0;i<nFFT;i++)
     {
      Data21[i].opEqual(F.Output[i]);//  Data21   F.Output;
      cData[i].conjugate(Data21[i]); //     Data21
      Data21[i].opMultEq(cData[i]);  //      
                                     //  ,     
     }
   F1.setCFFT(Data21,Data22);   // -   F1   
   F1.InverseT(nFFT);           //    
   getComplexArr(rA,F1.Output); //   double  
                                //      
   for(int i=0;i<nLags+1;i++)
     {
      ACF[i]=rA[i][0];          //    ACF  
                                // 
      ACF[i]=ACF[i]/rA[0][0];   //   
     }
//    
   double sigmaNMA;
   sigmaNMA=sqrt(1.0/ArraySize(res)); //   
   ArrayResize(bounds,2);
   double numSTD[2]={2,-2};           //     
   for(int i=0;i<2;i++)
      bounds[i]=sigmaNMA*numSTD[i];   //     
   ArrayResize(lags,nLags+1);
   for(int i=0;i<nLags+1;i++)
      lags[i]=i;                      // 
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double mean(double &X[])
// 
  {
   int len=ArraySize(X);
   double sum=0;
   for(int g=0;g<len;g++)
      sum+=X[g];
   return(sum/len);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int nextpow2(int n)
//    p,  2^p >= n
  {
   double F=0;
   double E=0;
   log2(F,E,n);
   if(F==0.5)
      E-=1; //E=E-1; 
   if(!MathIsValidNumber(F))
      E=F;
   return(E);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void log2(double &F,double &E,int n)
//    2
  {
   E = ceil(log(n)/log(2));
   F = n/pow(2,E);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void gser(double &gamser,double a,double x,double &gln)
/* Returns the incomplete gamma function P(a; x) evaluated
 by its series representation as gamser. */
  {
   int n;
   double sum,del,ap;
   gln=gammln(a);
   if(x<=0.0)
     {
      if(x<0.0) nrerror("x less than 0 in routine gser");
      gamser=0.0;
      return;
     }
   else
     {
      ap=a;
      del=sum=1.0/a;
      for(n=1;n<=ITMAX;n++)
        {
         ++ap;
         del *= x/ap;
         sum += del;
         if(fabs(del)<fabs(sum)*EPS)
           {
            gamser=sum*exp(-x+a*log(x)-(gln));
            return;
           }
        }
      nrerror("a too large, ITMAX too small in routine gser");
      return;
     }

  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void gcf(double &gammcf,double a,double x,double &gln)
/* Returns the incomplete gamma function Q(a; x) evaluated by its continued 
fraction representation as gammcf. Also returns ln (a) as gln.
*/
  {
   int i;
   double an,b,c,d,del,h;
   gln=gammln(a);
   b=x+1.0-a; //Set up for evaluating continued fraction by modified Lentz's method (x5.2) with b0 = 0.
   c=1.0/FPMIN;
   d=1.0/b;
   h=d;
   for(i=1;i<=ITMAX;i++)
     { //Iterate to convergence.
      an = -i*(i-a);
      b += 2.0;
      d=an*d+b;
      if(fabs(d)<FPMIN) d=FPMIN;
      c=b+an/c;
      if(fabs(c)<FPMIN) c=FPMIN;
      d=1.0/d;
      del=d*c;
      h *= del;
      if(fabs(del-1.0)<EPS) break;
     }
   if(i>ITMAX)
      nrerror("a too large, ITMAX too small in gcf");
   gammcf=exp(-x+a*log(x)-gln)*h; //Put factors in front.
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void nrerror(string error_text)
// 
  {
   Print("Numerical Recipes run-time error...\n");
   Print("%s\n",error_text);
   Print("...now exiting to system...\n");
   return;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double gammp(double a,double x)
//Returns the incomplete gamma function P(a; x).
  {
   double gamser,gammcf,gln;
   if(x<0.0 || a<=0.0) nrerror("Invalid arguments in routine gammp");
   if(x<(a+1.0))
     { //Use the series representation.
      gser(gamser,a,x,gln);
      return gamser;
     }
   else
     { //Use the continued fraction representation
      gcf(gammcf,a,x,gln);
      return 1.0-gammcf; //and take its complement.
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double gammln(double xx)
//Returns the value ln[(xx)] for xx > 0.
  {
//Internal arithmetic will be done in double precision, a nicety that 
//you can omit if five-figure accuracy is good enough.
   double x,y,tmp,ser;
   static double cof[6]=
     {
      76.18009172947146,-86.50532032941677,
      24.01409824083091,-1.231739572450155,
      0.1208650973866179e-2,-0.5395239384953e-5
     };
   int j;
   y=x=xx;
   tmp=x+5.5;
   tmp-=(x+0.5)*log(tmp);
   ser=1.000000000190015;
   for(j=0;j<=5;j++) ser+=cof[j]/++y;
   return -tmp+log(2.5066282746310005*ser/x);
  }
//+------------------------------------------------------------------+
void lbqtestInfo(bool &h1[],bool &h2[],double &rets[],double &retsq[],int &lags[])
  {
   CSymbolInfo m_smbinf; //   CSymbolInfo
   m_smbinf.Refresh(); m_smbinf.RefreshRates(); // refresh symbol info
   ENUM_TIMEFRAMES period=_Period;
   string periodStr;
   periodStr=EnumToString(period);

   m_smbinf.Name(Symbol());// /   
   string title="Q-  ",msg,
   str1="  ",str2="  ",
   str11="  .",str21="   .",
   Str1,Str2,
   line="--------------------------------------------",
   blanc="            ";
   double ACF1[],bounds1[],ACF2[],bounds2[];
   ArrayResize(ACF1,21);ArrayResize(ACF2,21);
   int maxLags=20;
   autocorr(ACF1,bounds1,lags,rets,maxLags);
   autocorr(ACF2,bounds2,lags,retsq,maxLags);

   if(h1[2]) Str1=str1;//    
   else Str1=str2;       //    
   if(h2[2]) Str2=str1;//     
   else Str2=str2;       //     

   msg=""+m_smbinf.Name()+", "+periodStr+"\n"
       +line+"\n"+Str1+"\n"+str11+"\n"+Str2+"\n"+str21
       +"\n"+line
       +"\n"+"# "+blanc+""+blanc+""
       ; //
   
   double acf1[]; ArrayResize(acf1,20);
   double acf2[]; ArrayResize(acf2,20);
   double acf1_b[]; ArrayResize(acf1_b,20); //   acf1
   double acf2_b[]; ArrayResize(acf2_b,20); //   acf2
   
   double ACF1_abs[]; ArrayResize(ACF1_abs,21); //   ACF1
   double ACF2_abs[]; ArrayResize(ACF2_abs,21); //   ACF2
   double k1; //   ACF1   
   double k2; //   ACF2   
   for(int i=0;i<21;i++)
     {
      ACF1_abs[i] = fabs(ACF1[i]);
      ACF2_abs[i] = fabs(ACF2[i]);
     }
   k1=ACF1_abs[ArrayMinimum(ACF1_abs)];
   k1=ceil(log(k1)/log(10));
   k1=1/pow(10,k1);
   k2=ACF2_abs[ArrayMinimum(ACF2_abs)];
   k2=ceil(log(k2)/log(10));
   k2=1/pow(10,k2);

   k2=(k2<1e4)?1e4:k2; //     
   k1=(k1<1e4)?1e4:k1; //     

   for(int i=9,k=0;i<21;i+=5,k++)
      msg=msg+"\n"+"     "+lags[i+1]+"   "+blanc+h1[k]+"   "+blanc+h2[k];
   for(int i=1;i<21;i++)
     {
      acf1[i-1]=k1*ACF1[i]; //todo
      acf2[i-1]=k2*ACF2[i];
     }
   for(int i=0;i<2;i++)
     {
      bounds1[i]=k1*bounds1[i];
      bounds2[i]=k2*bounds2[i];
     }
   for(int i=0;i<20;i++) //   
     {
      if(acf1[i]>0)acf1_b[i]=bounds1[0];
      else acf1_b[i]=bounds1[1];
      if(acf2[i]>0)acf2_b[i]=bounds2[0];
      else acf2_b[i]=bounds2[1];
     }
//display Google chart2
   CBarChart chart2;
   chart2.Attach(0,"Chart2");
// 
   chart2.SetSize(X_size,Y_size);
   int first_line2=chart2.AddLine(acf2,Blue,0,"ACF");
   if(first_line2<0)
      Print("Add first line error"+DoubleToString(GetLastError(),0));
   int second_line2=chart2.AddLine(acf2_b,Red,0,"Bounds");
   if(second_line2<0)
      Print("Add first line error"+DoubleToString(GetLastError(),0));
// 
   chart2.SetAxis(DIAGRAM_AXIS_BOTTOM|DIAGRAM_AXIS_LEFT,-1,0,0,5,3);
// 
   chart2.SetGrouped(true,0.1,0.25);
//
   chart2.SetGrid(20.0,10.0);
//
   chart2.ShowLegend();
//
   chart2.SetTitle("ACF for squared retuns, "+m_smbinf.Name()+", "+periodStr,Green,12);
// 
   chart2.SetFill(Linen,Silver);
   chart2.SetAxisLabels("Lags","","1e"+DoubleToString(log10(k2),0),"",Black,12);
   chart2.GetChart();
//display Google chart1
   CBarChart chart1;
   chart1.Attach(0,"Chart1");
// 
   chart1.SetSize(X_size,Y_size);
   int first_line1=chart1.AddLine(acf1,Blue,0,"ACF");
   if(first_line1<0)
      Print("Add first line error"+DoubleToString(GetLastError(),0));
   int second_line1=chart1.AddLine(acf1_b,Red,0,"Bounds");
   if(second_line1<0)
      Print("Add first line error"+DoubleToString(GetLastError(),0));
// 
   chart1.SetAxis(DIAGRAM_AXIS_BOTTOM|DIAGRAM_AXIS_LEFT,-1,0,0,5,3);
// 
   chart1.SetGrouped(true,0.1,0.25);
//
   chart1.SetGrid(20.0,10.0);
//
   chart1.ShowLegend();
//
   chart1.SetTitle("ACF for retuns, "+m_smbinf.Name()+", "+periodStr,Green,12);
// 
   chart1.SetFill(Linen,Silver);
   chart1.SetAxisLabels("Lags","","1e"+DoubleToString(log10(k1),0),"",Black,12);
   chart1.GetChart();
//  Q-  
   finish=GetTickCount()-start;  //   
   msg=msg+"\n"+line+"\n"+"   "+DoubleToString(finish,0)+" msec";
   MessageBox(msg,title,MB_OK|MB_ICONINFORMATION);
  }
//+------------------------------------------------------------------+
bool any(bool &h[])
//,    "true"   
  {
   bool B=false;
   for(int i=0;i<ArraySize(h);i++)
     {
      if(h[i])
        {
         B=true;
         break;
        }
      else
         continue;
     }
   return B;
  }
//+------------------------------------------------------------------+
